import Hapi from '@hapi/hapi';
import Pino from 'hapi-pino';
import debug from 'debug';
import toMultiaddr from 'uri-to-multiaddr';
import routes from './routes/index.js';
const LOG = 'ipfs:http-gateway';
const LOG_ERROR = 'ipfs:http-gateway:error';
function hapiInfoToMultiaddr(info) {
  let hostname = info.host;
  let uri = info.uri;
  if (hostname.includes(':') && !hostname.startsWith('[')) {
    hostname = `[${ hostname }]`;
    uri = uri.replace(`://${ info.host }`, `://${ hostname }`);
  }
  return toMultiaddr(uri);
}
async function serverCreator(serverAddrs, createServer, ipfs) {
  serverAddrs = serverAddrs || [];
  serverAddrs = Array.isArray(serverAddrs) ? serverAddrs : [serverAddrs];
  const servers = [];
  for (const address of serverAddrs) {
    const addrParts = address.split('/');
    const server = await createServer(addrParts[2], addrParts[4], ipfs);
    await server.start();
    server.info.ma = hapiInfoToMultiaddr(server.info);
    servers.push(server);
  }
  return servers;
}
export class HttpGateway {
  constructor(ipfs) {
    this._ipfs = ipfs;
    this._log = Object.assign(debug(LOG), { error: debug(LOG_ERROR) });
    this._gatewayServers = [];
  }
  async start() {
    this._log('starting');
    const ipfs = this._ipfs;
    const config = await ipfs.config.getAll();
    const addresses = config.Addresses || {
      Swarm: [],
      Gateway: []
    };
    const gatewayAddrs = addresses?.Gateway || [];
    this._gatewayServers = await serverCreator(gatewayAddrs, this._createGatewayServer, ipfs);
    this._log('started');
  }
  async _createGatewayServer(host, port, ipfs) {
    const server = Hapi.server({
      host,
      port,
      routes: {
        cors: true,
        response: { emptyStatusCode: 200 }
      }
    });
    server.app.ipfs = ipfs;
    await server.register({
      plugin: Pino,
      options: {
        prettyPrint: Boolean(debug.enabled(LOG)),
        logEvents: [
          'onPostStart',
          'onPostStop',
          'response',
          'request-error'
        ],
        level: debug.enabled(LOG) ? 'debug' : debug.enabled(LOG_ERROR) ? 'error' : 'fatal'
      }
    });
    server.route(routes.gateway);
    return server;
  }
  async stop() {
    this._log('stopping');
    const stopServers = servers => Promise.all((servers || []).map(s => s.stop()));
    await Promise.all([stopServers(this._gatewayServers)]);
    this._log('stopped');
  }
}